/*
=========================================================================
	CREATURE.H
    Interface to the Creature class (PC, NPC, and Monster entities).
	Copyright (c) 2004-2007 by Daniel R. Collins. All rights reserved.
  May be distributed under the terms of GPL version 2 (see Copying.txt).
=========================================================================
*/

#ifndef _CREATURE_H
#define _CREATURE_H

#include "dice.h"
#include "weapon.h"
#include <list>
#include <string>
using namespace std;

typedef list<Weapon> WeaponList;


class Creature {
public:
  // Enumerations
  enum StatEnum { CR, AC, HP, SR, AL, init, speed, size, numStats };
  enum AbilityEnum { Str, Dex, Con, Int, Wis, Cha, numAbilities };
  enum FaceEnum { Height, Width, Reach, numFaces };
  enum SaveEnum { Fort, Ref, Will, numSaves };

  // Constructors
  Creature (const char *data);
  ~Creature (void);

  // Accessors
  int getStat (int statNum);
  int getFace (int faceNum);
  int getSave (int saveNum);
  int getAbility (int abilityNum);
  int getNumWeapons (int category);
  int getNumAttacks (int category);
  Weapon *getWeapon (int category, int num);

  // Mutators
  int rollHitPoints (void);

  // Combat accessors
  int getHitPoints (void);
  bool isDown (void);
  bool isDead (void);

  // Combat actions
  int attack (int category, int num, Creature *target, int atkMod);
  int takeDamage (int amount);

protected:
  // Helper functions
  WeaponList *getWeaponList (int category);
  void readAttacks (const char *data);
  void readWeapon (const char *data, int category);
  void readAlignment (const char *data);

private:
  // String members
  string name, type;
  string specialAttacks, specialQualities;

  // Data members
  Dice HitDice;
  int stat[numStats], ability[numAbilities];
  int face[numFaces], save[numSaves];
  WeaponList meleeAttacks, rangedAttacks;

  // TODO: Skills & Feats
  // TODO: Type, Subtype(s)

  // Combat variables
  int damage, subdual;
};


// TODO: ActiveCreature, child class: combat functions, damage, subdual.
// TODO: MonsterType, child class: treasure, number, etc.

#endif

